@@ -57,12 +57,23 @@ if (report_diffs || build_table) {
   }
 }
 
+# Some tests make plot.ly HTTP requests and require a valid user account
+# (see test-plotly-filename.R). For security reasons, these tests should be 
+# skipped on pull requests (the .travis.yml file uses encrypted credentials
+# & encrypted environment vars cannot be accessed on pull request builds)
+skip_on_pull_request <- function() {
+  if (!grepl("^[0-9]+$", Sys.getenv("TRAVIS_PULL_REQUEST"))) {
+    return(invisible(TRUE))
+  }
+  skip("Can't test plot.ly API calls on a pull request")
+}
+
 # This function is called within testthat/test-*.R files.
 # It takes a ggplot or plotly object as input, and it returns a figure
 # object (aka the data behind the plot).
 save_outputs <- function(gg, name) {
   print(paste("Running test:", name))
-  p <- plotly_build(gg)
+  p <- plotly_build(gg)$x[c("data", "layout")]
   has_diff <- if (report_diffs) {
     # save a hash of the R object
     plot_hash <- digest::digest(p)
@@ -73,14 +84,8 @@ save_outputs <- function(gg, name) {
     !isTRUE(plot_hash == test_info$hash)
   } else FALSE
   if (has_diff || build_table) {
-    # hack to transfer workspace to the other R session
-    rs_assign <- function(obj, name) RSassign(conn, obj, name)
-    res <- mapply(rs_assign, mget(ls()), ls())
-    # also need to transfer over the plotly environment to enable NSE
-    res <- RSassign(conn, plotly:::plotlyEnv, "plotlyEnv")
-    res <- RSeval(conn, "unlockBinding('plotlyEnv', asNamespace('plotly'))")
-    res <- RSeval(conn, "assign('plotlyEnv', plotlyEnv, pos = asNamespace('plotly'))")
-    pm <- RSeval(conn, "tryCatch(plotly::plotly_build(gg), error = function(e) 'plotly build error')")
+    RSassign(conn, gg)
+    pm <- RSeval(conn, "tryCatch(plotly::plotly_build(gg)$x[c('data', 'layout')], error = function(e) e$message)")
     if (build_table) {
       # save pngs of ggplot
       filename <- paste0(gsub("\\s+", "-", name), ".png")
